# Deinitions for some stx:libbasic classes
#

import gdb
import stx.gdb.objects


class Object( stx.gdb.objects.__instance ):
    None

class UndefinedObject ( stx.gdb.objects.__instance ):
    def __init__(self, val):
        #stx.support.Decorator.__init__(self, val)
        #object.__setattr__(self, "_klass", None)
        self._obj = None
        self._klass = None

    def display_value(self):
        '''
        Return a human-readable represenation of object's value, if sensible.
        FOr instance, for integer or string it may be its respective value
        '''
        return 'nil'

    def display_string(self):
        return 'nil'

    @property
    def address(self):
        return 0
    
    @property
    def klass(self):
        '''Return this object's class as an __instance value'''
        if (self._klass == None):
            self._klass = stx.gdb.objects.create(gdb.parse_and_eval('UndefinedObject'))
        return self._klass

    @property
    def size(self):
        return 0

    @property
    def size_b(self):
        return 0

nil = UndefinedObject(None)


class SmallInteger ( stx.gdb.objects.__instance ):
    def display_value(self):
        '''
        Return a human-readable represenation of object's value, if sensible.
        FOr instance, for integer or string it may be its respective value
        '''
        return self.address >> 1

    @property
    def klass(self):
        '''Return this object's class as an __instance value'''
        if (self._klass == None):
            self._klass = Class(gdb.parse_and_eval('__GLOBAL_GET_BY_NAME("SmallInteger")'))
        return self._klass

    @property
    def size(self):
        return 0

    @property
    def size_b(self):
        return 0

    def display_string(self):
        value = self.display_value()
        return "0x%08x (SmallInteger %s)" % ( self.address , value )




class Behavior ( Object ):
    @property
    def superclass(self):
        if (self._superclass == None):
            self._superclass = stx.gdb.objects.create(self._obj['i_instvars'][0])
        return self._superclass


class ClassDescription ( Behavior ):
    @property 
    def instvars(self):
        return []

        
class Class ( ClassDescription ):
    @property 
    def name(self):
        try:
            return self._obj.cast(gdb.lookup_type('struct __instance').pointer())['i_instvars'][6].cast(gdb.lookup_type('struct __Symbol').pointer())['s_element'].cast(gdb.lookup_type('char').pointer()).string()
        except:
            return "** invalid class name **"

class Metaclass ( ClassDescription ):
    @property 
    def name(self):
        try:
            return '%s class' % self._obj.cast(gdb.lookup_type('struct __instance').pointer())['i_instvars'][6].cast(gdb.lookup_type('struct __instance').pointer())['i_instvars'][6].cast(gdb.lookup_type('struct __Symbol').pointer())['s_element'].cast(gdb.lookup_type('char').pointer()).string()
        except:
            return "** invalid class x.name ** class"


class Symbol ( Object ):
    def __init__(self, val):
        self._obj = val.cast(gdb.lookup_type('struct __Symbol').pointer())
        self._klass = None


    def display_value(self):
        '''
        Return a human-readable represenation of object's value, if sensible.
        For instance, for integer or string it may be its respective value
        '''
        return '#' + self._obj['s_element'].cast(gdb.lookup_type('char').pointer()).string()

